﻿using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using Telerik.Windows.Data;
using DomainServicesMasterDetailsMVVM.Web;

namespace DomainServicesMasterDetailsMVVM
{
	public class MyViewModel : INotifyPropertyChanged
	{
		public event PropertyChangedEventHandler PropertyChanged;

		private readonly NorthwindContext ordersContext;
		private readonly QueryableDomainServiceCollectionView<Order> ordersView;
		private readonly NorthwindContext orderDetailsContext;
		private readonly QueryableDomainServiceCollectionView<Order_Detail> orderDetailsView;
		private readonly FilterDescriptor orderDetailsFilter;

		private Order selectedOrder;
		
		public Order SelectedOrder
		{
			get { return this.selectedOrder; }
			set
			{
				if (value != this.selectedOrder)
				{
					this.selectedOrder = value;
					
					if (this.orderDetailsView.IsLoading)
					{
						this.orderDetailsView.CancelLoad();
					}
					this.orderDetailsFilter.Value = this.SelectedOrder.OrderID;
					this.orderDetailsView.Load();
					
					this.OnPropertyChanged("SelectedOrder");
				}
			}
		}

		public IEnumerable<Order> Orders
		{
			get
			{
				return this.ordersView;
			}
		}

		public IEnumerable<Order_Detail> OrderDetails
		{
			get
			{
				return this.orderDetailsView;
			}
		}

		public bool AreOrdersBusy
		{
			get
			{
				return this.ordersView.IsBusy;
			}
		}

		public bool AreOrderDetailsBusy
		{
			get
			{
				return this.orderDetailsView.IsBusy;
			}
		}

		public MyViewModel()
		{
			this.ordersContext = new NorthwindContext();
			this.ordersView = new QueryableDomainServiceCollectionView<Order>(this.ordersContext
				, this.ordersContext.GetOrdersQuery());
			this.ordersView.PropertyChanged += this.OnOrdersViewPropertyChanged;
			this.ordersView.AutoLoad = true;
			
			this.orderDetailsContext = new NorthwindContext();
			this.orderDetailsView = new QueryableDomainServiceCollectionView<Order_Detail>(
				this.orderDetailsContext
				, this.orderDetailsContext.GetOrder_DetailsQuery());
			this.orderDetailsFilter = new FilterDescriptor("OrderID", FilterOperator.IsEqualTo, null);
			this.orderDetailsView.FilterDescriptors.Add(this.orderDetailsFilter);
			this.orderDetailsView.PropertyChanged += this.OnOrderDetailsViewPropertyChanged;
		}

		private void OnOrdersViewPropertyChanged(object sender, PropertyChangedEventArgs e)
		{
			if (e.PropertyName == "IsBusy")
			{
				this.OnPropertyChanged("AreOrdersBusy");
			}
		}
		
		private void OnOrderDetailsViewPropertyChanged(object sender, PropertyChangedEventArgs e)
		{
			if (e.PropertyName == "IsBusy")
			{
				this.OnPropertyChanged("AreOrderDetailsBusy");
			}
		}
		
		protected virtual void OnPropertyChanged(PropertyChangedEventArgs args)
		{
			var handler = this.PropertyChanged;
			if (handler != null)
			{
				handler(this, args);
			}
		}

		private void OnPropertyChanged(string propertyName)
		{
			this.OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
		}
	}
}
